/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import java.util.Collection;
import java.util.Collections;

import lombok.AccessLevel;
import lombok.NoArgsConstructor;

@NoArgsConstructor(access = AccessLevel.PRIVATE)
public class UserRole {

	public static final String EINHEITLICHER_ANSPRECHPARTNER = "EINHEITLICHER_ANSPRECHPARTNER";
	public static final String VERWALTUNG_POSTSTELLE = "VERWALTUNG_POSTSTELLE";
	public static final String VERWALTUNG_USER = "VERWALTUNG_USER";
	public static final String VERWALTUNG_LOESCHEN = "VERWALTUNG_LOESCHEN";

	private static final RoleHierarchy HIERARCHY = RoleHierarchy.builder()
			.roleName(VERWALTUNG_LOESCHEN)
			.subRole(RoleHierarchy.builder()
					.roleName(VERWALTUNG_USER)
					.build())
			.build();

	public static boolean containsWithinRoleHierarchy(Collection<String> userRoles, String roleName) {
		return !Collections.disjoint(userRoles, HIERARCHY.getFlattenedHierarchy(roleName));
	}

}