/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user.common.migration;

import java.util.Map;

import org.bson.Document;

import com.mongodb.client.MongoCollection;
import com.mongodb.client.MongoDatabase;

import io.mongock.api.annotations.ChangeUnit;
import io.mongock.api.annotations.Execution;
import io.mongock.api.annotations.RollbackExecution;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@ChangeUnit(id = "2024-09-22 12:00:00 OZG-3539", order = "M001", author = "sbergandy", systemVersion = "1")
public class M001_SetNewAndMigrateUserSettings {

	static final String USER_COLLECTION_NAME = "User";
	static final String USER_SETTINGS_FIELD = "userSettings";
	static final String NOTIFICATIONS_SEND_FOR_FIELD = "notificationsSendFor";
	static final String VORGANG_CREATED_FIELD = "vorgangCreated";
	static final String VORGANG_ASSIGNED_TO_USER_FIELD = "vorgangAssignedToUser";
	static final String POSTFACH_NACHRICHT_FROM_ANTRAGSTELLER_FIELD = "postfachNachrichtFromAntragsteller";
	static final String WIEDERVORLAGE_DUE_TODAY = "wiedervorlageDueToday";

	private final MongoDatabase mongoDatabase;

	@Execution
	public void migrationMethod() {
		var userCollection = mongoDatabase.getCollection(USER_COLLECTION_NAME);
		migrateNotificationsSendForALL(userCollection);
		migrateNotificationsSendForNONE(userCollection);
		migrateEmptySettings(userCollection);
	}

	private void migrateNotificationsSendForALL(MongoCollection<Document> userCollection) {
		userCollection.updateMany(
				new Document(Map.of(buildFieldPath(USER_SETTINGS_FIELD, NOTIFICATIONS_SEND_FOR_FIELD), "ALL")),
				new Document(Map.of("$set", Map.of(USER_SETTINGS_FIELD,
						Map.of(NOTIFICATIONS_SEND_FOR_FIELD, "ALL",
								VORGANG_CREATED_FIELD, true,
								VORGANG_ASSIGNED_TO_USER_FIELD, true,
								POSTFACH_NACHRICHT_FROM_ANTRAGSTELLER_FIELD, true,
								WIEDERVORLAGE_DUE_TODAY, true)))));
	}

	private void migrateNotificationsSendForNONE(MongoCollection<Document> userCollection) {
		userCollection.updateMany(
				new Document(Map.of(buildFieldPath(USER_SETTINGS_FIELD, NOTIFICATIONS_SEND_FOR_FIELD), "NONE")),
				new Document(Map.of("$set", Map.of(USER_SETTINGS_FIELD,
						Map.of(NOTIFICATIONS_SEND_FOR_FIELD, "NONE",
								VORGANG_CREATED_FIELD, false,
								VORGANG_ASSIGNED_TO_USER_FIELD, true,
								POSTFACH_NACHRICHT_FROM_ANTRAGSTELLER_FIELD, true,
								WIEDERVORLAGE_DUE_TODAY, true)))));
	}

	private void migrateEmptySettings(MongoCollection<Document> userCollection) {
		userCollection.updateMany(
				new Document(Map.of(buildFieldPath(USER_SETTINGS_FIELD, NOTIFICATIONS_SEND_FOR_FIELD), Map.of("$exists", false))),
				new Document(Map.of("$set", Map.of(USER_SETTINGS_FIELD,
						Map.of(NOTIFICATIONS_SEND_FOR_FIELD, "NONE",
								VORGANG_CREATED_FIELD, false,
								VORGANG_ASSIGNED_TO_USER_FIELD, true,
								POSTFACH_NACHRICHT_FROM_ANTRAGSTELLER_FIELD, true,
								WIEDERVORLAGE_DUE_TODAY, true)))));
	}

	@RollbackExecution
	public void rollback() {
		// kein rollback implementiert
	}

	private String buildFieldPath(String... fields) {
		return String.join(".", fields);
	}
}
